@@ -24,7 +24,7 @@ import subprocess
 import sys
 import time
 
-from upmplgutils import uplog
+from upmplgutils import uplog, getOptionValue
 import uprclinit
 
 
@@ -36,7 +36,7 @@ def _initconfdir(confdir, topdirs):
         if os.path.exists(confdir):
             raise Exception("Exists and not directory: %s" % confdir)
         os.mkdir(confdir)
-        
+
     datadir = os.path.dirname(__file__)
     uplog("datadir: %s" % datadir)
 
@@ -44,26 +44,39 @@ def _initconfdir(confdir, topdirs):
         dst = os.path.join(confdir, fn)
         src = os.path.join(datadir, "rclconfig-" + fn)
         shutil.copyfile(src, dst)
+    try:
+        with open(os.path.join(confdir, "backends"), "wb") as f:
+            pass
+    except Exception as err:
+        uplog(f"Failed truncating backends: {err}")
+
+    # Update "stored" section of the "fields" file according to minim config: some custom
+    # tags may nnot be in our predefined list
+    fields = uprclinit.allMinimTags()
+    dst = os.path.join(confdir, "fields")
+    fldconf = conftree.ConfSimple(dst, False, False)
+    for fld in fields:
+        fldconf.set(fld, "", "stored")
 
     exclpats = uprclinit.g_minimconfig.getexcludepatterns()
 
-    userconfig = uprclinit.g_upconfig.get("uprclconfrecolluser")
+    userconfig = getOptionValue("uprclconfrecolluser")
     if not userconfig:
         userconfig = os.path.join(confdir, "recoll.conf.user")
     if os.path.exists(userconfig):
         userconfdata = open(userconfig, "rb").read()
     else:
-        userconfdata = b''
-        
+        userconfdata = b""
+
     path = os.path.join(confdir, "recoll.conf")
     f = open(path, "wb")
-    f.write(b"topdirs = %s\n"% topdirs.encode(locale.getpreferredencoding()))
+    f.write(b"topdirs = %s\n" % topdirs.encode(locale.getpreferredencoding()))
     f.write(b"idxabsmlen = 0\n")
     f.write(b"loglevel = 2\n")
-    #f.write(b"idxlogfilename = /tmp/loguprcl.txt\n")
-    #f.write(b"pylogfilename = /tmp/logpyuprcl.txt\n")
+    # f.write(b"idxlogfilename = /tmp/loguprcl.txt\n")
+    # f.write(b"pylogfilename = /tmp/logpyuprcl.txt\n")
     f.write(b"noaspell = 1\n")
-    f.write(b"nomd5types = rclaudio rclimg\n")
+    f.write(b"nomd5types = rclaudio rclaudio.py rclimg rclimg.py\n")
     f.write(b"testmodifusemtime = 1\n")
     f.write(b"idxmetastoredlen = 20000\n")
     # We'd like to set this to zero, but this triggers the creation of
@@ -72,8 +85,13 @@ def _initconfdir(confdir, topdirs):
     # chert backend disabled, even if XAPIAN_HAS_CHERT_BACKEND is set
     # in version.h
     f.write(b"indexStoreDocText = 1\n")
-    f.write(b"audiotagfixerscript = %b\n" %
-            os.path.join(datadir, "minimtagfixer.py").encode('utf-8'))
+    f.write(
+        b"audiotagfixerscript = %b\n" % os.path.join(datadir, "minimtagfixer.py").encode("utf-8")
+    )
+    f.write(b"indexallfilenames = 0\n")
+    f.write(
+        b"noContentSuffixes+ = .tiff .tif .xcf .bmp .xpm .svg .djvu\n"
+    )
     if exclpats:
         f.write(b"skippedNames+ = " + exclpats.encode("utf-8") + b"\n")
     else:
@@ -86,6 +104,7 @@ def _initconfdir(confdir, topdirs):
 _idxproc = None
 _lastidxstatus = None
 
+
 def runindexer(confdir, topdirs, rebuild=False):
     global _idxproc, _lastidxstatus
     if _idxproc is not None:
@@ -93,8 +112,8 @@ def runindexer(confdir, topdirs, rebuild=False):
 
     _initconfdir(confdir, topdirs)
 
-    cf = conftree.ConfSimple(os.path.join(confdir, "recoll.conf"), readonly = False)
-    td = cf.get("topdirs", '')
+    cf = conftree.ConfSimple(os.path.join(confdir, "recoll.conf"), readonly=False)
+    td = cf.get("topdirs", "")
     if td != topdirs:
         cf.set("topdirs", topdirs)
 
@@ -105,6 +124,7 @@ def runindexer(confdir, topdirs, rebuild=False):
     else:
         _idxproc = subprocess.Popen(["recollindex", "-c", confdir])
 
+
 def indexerdone():
     global _idxproc, _lastidxstatus
     if _idxproc is None:
@@ -115,13 +135,13 @@ def indexerdone():
     _idxproc = None
     return True
 
+
 def indexerstatus():
     return _lastidxstatus
-    
 
 
 # Only used for testing
-if __name__ == '__main__':
+if __name__ == "__main__":
     if len(sys.argv) != 3:
         print("Usage: uprclindex.py <confdir> <topdirs>", file=sys.stderr)
         sys.exit(1)
@@ -131,4 +151,4 @@ if __name__ == '__main__':
             uplog("Indexing done, status: %d" % indexerstatus())
             sys.exit(0)
         uplog("Waiting for indexer")
-        time.sleep(1);
+        time.sleep(1)
