@@ -20,7 +20,6 @@ import time
 import re
 from recoll import recoll
 
-from timeit import default_timer as timer
 from uprclutils import audiomtypes, docfolder, uplog
 import uprclutils
 import uprclinit
@@ -179,7 +178,8 @@ def _createTagTables(cursor, tgnm):
         cursor.execute("DROP TABLE " + _junctb(tgnm))
     except:
         pass
-    stmt = "CREATE TABLE " + _junctb(tgnm) + " (docidx INT, " + _clid(tgnm) + " INT)"
+    stmt = f"CREATE TABLE {_junctb(tgnm)} (docidx INT, {_clid(tgnm)} INT, " \
+        f"UNIQUE(docidx, {_clid(tgnm)}))"
     cursor.execute(stmt)
 
 
@@ -231,6 +231,18 @@ def _prepareTags(conn):
         rclfld = _coltorclfield[tb] if tb in _coltorclfield else tb
         uplog(f"prepareTags: using rclfield [{rclfld}] for sql [{tb}]")
         tabtorclfield.append((tb, rclfld))
+        # Special case "artist" because we also want "composer" and "conductor" in there.  This is
+        # so that recoll doc records will be created by _artiststorecoll(), and search for artist by
+        # title will find them. This is a bit dubious because, if there are Composer or Conductor
+        # indexes, we are duplicating data, and the user probably does not want to find composers in
+        # the artist list. The other possible approach would (tbchecked?) be to use itemtags tables
+        # but this would need additional code in uprcltags.py to duplicate the direntryforartid()
+        # method and the code in uprclsearch which calls it + maybe some of the code which links
+        # artists to albums and tracks probably? may worth a try anyway, it would be nicer if not
+        # too complicated.
+        if tb == "artist":
+            tabtorclfield.append((tb, "composer"))
+            tabtorclfield.append((tb, "conductor"))
 
     for nm in itemtags:
         if nm not in _alltagtotable:
@@ -246,7 +258,7 @@ def _prepareTags(conn):
 
 # Insert new value if not existing, return rowid of new or existing row
 def _auxtableinsert(conn, tb, value):
-    # uplog("_auxtableinsert [%s] -> [%s]" % (tb, value))
+    # uplog(f"_auxtableinsert {tb} -> {value}")
     c = conn.cursor()
     stmt = f"SELECT {_clid(tb)} FROM  {tb} WHERE value = ?"
     c.execute(stmt, (value,))
@@ -283,7 +295,7 @@ _folderdnumexp = re.compile(_folderdnumre, flags=re.IGNORECASE)
 # The albums table is special, can't use auxtableinsert()
 def _maybecreatealbum(conn, doc):
     c = conn.cursor()
-    folder = docfolder(doc).decode("utf-8", errors="replace")
+    folder = docfolder(doc)
 
     album = doc["album"]
     if not album:
@@ -370,7 +382,8 @@ def _updatealbartistlist(conn, album_id, rowids):
 # Setting album covers needs to wait until we have scanned all tracks so that we can select
 # a consistant embedded art (first track in path order), as recoll scanning is in unsorted
 # directory order.
-def _setalbumcovers(conn, rcldocs):
+def _setalbumcovers(conn, folders):
+    rcldocs = folders.rcldocs()
     c = conn.cursor()
     c.execute("""SELECT album_id,albtitle FROM albums""")
     for r in c:
@@ -382,10 +395,8 @@ def _setalbumcovers(conn, rcldocs):
         for r1 in c1:
             docidx = r1[0]
             doc = rcldocs[docidx]
-            arturi = uprclutils.docarturi(
+            arturi = folders.docarturi(
                 doc,
-                uprclinit.getHttphp(),
-                uprclinit.getPathPrefix(),
                 preferfolder=True,
                 albtitle=albtitle,
             )
@@ -446,7 +457,7 @@ def _albumstorecoll(conn, rcldb):
             doc["albumartist"] = r[5]
             doc["artist"] = r[5]
         doc["url"] = "file://" + r[1]
-        # uplog(f"_albumstorecoll: indexing album {doc['album']}")
+        # uplog(f"_albumstorecoll: indexing album {doc['title']}")
         rcldb.addOrUpdate(udi, doc)
 
 
@@ -603,8 +614,9 @@ def parsedate(dt):
 
 # Create the db and fill it up with the values we need, taken out of
 # the recoll records list
-def recolltosql(conn, rcldocs):
-    start = timer()
+def recolltosql(conn, folders):
+    rcldocs = folders.rcldocs()
+    start = time.time()
 
     _createsqdb(conn)
     tabtorclfield = _prepareTags(conn)
@@ -640,10 +652,7 @@ def recolltosql(conn, rcldocs):
 
         trackno = _tracknofordoc(doc)
 
-        if doc["url"].find("file://") == 0:
-            path = doc["url"][7:]
-        else:
-            path = ""
+        path = uprclutils.docpath(doc)
 
         # Misc tag values:
         # Set base values for column names, values list, placeholders. Done this way because we used
@@ -667,14 +676,14 @@ def recolltosql(conn, rcldocs):
             if not value:
                 continue
             # rclaudio.py concatenates multiple values, using " | " as separator.
-            valuelist = value.split(" | ")
+            valuelist = set(value.split(" | "))
             rowids = set()
             for value in valuelist:
                 # Possibly insert in appropriate table (if value not already there), and
                 # insert record in junction table for the corresponding field.
                 rowid = _auxtableinsert(conn, tb, value)
                 rowids.add(rowid)
-                stmt = f"INSERT INTO {_junctb(tb)}(docidx, {_clid(tb)}) VALUES (?, ?)"
+                stmt = f"INSERT OR IGNORE INTO {_junctb(tb)}(docidx, {_clid(tb)}) VALUES (?, ?)"
                 jvals = [docidx, rowid]
                 # uplog(f"EXECUTING {stmt} values {jvals}")
                 c.execute(stmt, jvals)
@@ -683,15 +692,23 @@ def recolltosql(conn, rcldocs):
         # Create the main record in the tracks table.
         stmt = "INSERT INTO tracks(" + ",".join(columns) + ") VALUES(" + ",".join(placehold) + ")"
         c.execute(stmt, values)
-
+    
     ## End Big doc loop
 
+    #t1 = time.time()
+    #uplog(f"recolltosql: docwalk: {t1-start:.1f} Seconds")
     _setalbumartists(conn)
-    _setalbumcovers(conn, rcldocs)
+    _setalbumcovers(conn, folders)
+    #t2 = time.time()
+    #uplog(f"recolltosql: setalbumxx: {t2-t1:.1f} Seconds")
     _createmergedalbums(conn)
+    #t1 = time.time()
+    #uplog(f"recolltosql: createmergedalbums: {t1-t2:.1f} Seconds")
     conn.commit()
-    end = timer()
+    #t2 = time.time()
+    #uplog(f"recolltosql: commit: {t2-t1:.1f} Seconds")
     rcldb = recoll.connect(confdir=uprclinit.getRclConfdir(), writable=True)
     _albumstorecoll(conn, rcldb)
     _artiststorecoll(conn, rcldb)
+    end = time.time()
     uplog(f"recolltosql: processed {totcnt} docs in {end-start:.1f} Seconds")
