@@ -45,17 +45,17 @@ MPDCli::MPDCli(const string& host, int port, const string& pass)
     std::unique_lock<std::mutex> lock(m_mutex);
 
     regcomp(&m_tpuexpr, "^[[:alpha:]]+://.+", REG_EXTENDED|REG_NOSUB);
-    g_config->get("onstart", m_onstart);
-    g_config->get("onplay", m_onplay);
-    g_config->get("onpause", m_onpause);
-    g_config->get("onstop", m_onstop);
+    getOptionValue("onstart", m_onstart);
+    getOptionValue("onplay", m_onplay);
+    getOptionValue("onpause", m_onpause);
+    getOptionValue("onstop", m_onstop);
     string scratch;
-    g_config->get("onvolumechange", scratch);
+    getOptionValue("onvolumechange", scratch);
     stringToStrings(scratch,  m_onvolumechange);
-    g_config->get("getexternalvolume", scratch);
+    getOptionValue("getexternalvolume", scratch);
     stringToStrings(scratch, m_getexternalvolume);
-    m_timeoutms = g_config->getInt("mpdtimeoutms", 2000);
-    m_externalvolumecontrol = g_config->getBool("externalvolumecontrol", false);
+    m_timeoutms = getIntOptionValue("mpdtimeoutms", 2000);
+    m_externalvolumecontrol = getBoolOptionValue("externalvolumecontrol", false);
 
     if (!openconn()) {
         return;
@@ -488,8 +488,7 @@ bool MPDCli::updStatus()
     m_stat.songelapsedms = mpd_status_get_elapsed_ms(mpds);
     m_stat.songlenms = mpd_status_get_total_time(mpds) * 1000;
     m_stat.kbrate = mpd_status_get_kbit_rate(mpds);
-    const struct mpd_audio_format *maf =
-        mpd_status_get_audio_format(mpds);
+    const struct mpd_audio_format *maf = mpd_status_get_audio_format(mpds);
     if (maf) {
         m_stat.bitdepth = maf->bits;
         m_stat.sample_rate = maf->sample_rate;
@@ -501,8 +500,7 @@ bool MPDCli::updStatus()
         m_stat.currentsong.rsrc.samplefreq = m_stat.sample_rate;
         m_stat.currentsong.rsrc.bitsPerSample = m_stat.bitdepth;
         m_stat.currentsong.rsrc.channels = m_stat.channels;
-
-        LOGDEB1("MPD AUDIO FORMAT: " <<  int(maf->sample_rate) << " samps/S " <<
+        LOGDEB1("mpd audioformat: " <<  int(maf->sample_rate) << " samps/S " <<
                 m_stat.kbrate << " kbits/S " << int(maf->bits) << " bits " <<
                 int(maf->channels) << " channels\n");
     } else {
@@ -768,10 +766,9 @@ bool MPDCli::setVolume(int volume, bool isMute)
         stringstream ss;
         ss << volume;
         args.push_back(ss.str());
-        if (ecmd.doexec1(args)) {
+        if (ecmd.doexec(args)) {
             // doexec returns the exit status, should be zero
-            LOGDEB("MPDCli::setVolume: " << m_onvolumechange[0] <<
-                   " failed " << volume << endl);
+            LOGDEB("MPDCli::setVolume: " << m_onvolumechange[0] << " failed " << volume << endl);
         }
     }
     m_stat.volume = volume;
